/*
	muFileListUtils.h
	MetroTools, Release 2.1
	Copyright (c) 1992-93, Metrosoft.  All rights reserved. 
*/

/*----------------------------------------------------------------------------
							FILE LIST UTILITIES
							
	This module contains utilities that handle retrieving lists of file paths,
	and the maintenance of those lists.
	
	filename: muFileListUtils.h
	author: Ron Miller
	date: 12/8/91
	
	modification history:
		7/26/93 RLM - modified for MetroKeys
		
------------------------------------------------------------------------------*/

#import <appkit/appkit.h>

/* list of file pathnames */
typedef char pathString[MAXPATHLEN+1];

typedef struct
	{
		int			nFiles;
		pathString	paths[0];
	} FileListRec, *FileListPtr;



/* file list utils */
void muSetHomeModuleDirectory( char *dirName );
/* this routine sets the default module directory that muGetHomeModulePath()
	work from.
*/

void muSetPublicModuleDirectory( char *dirName );
/* this routine sets the default module directory that muGetPublicModulePath()
	work from.
*/

FileListPtr muNewFileList( void );
/* this routine returns an allocated and initialized file list ptr. returns
	NULL if problem.
*/

BOOL muGetFileList( char *path, char *ext, FileListPtr *fp, BOOL noDups );
/* this routine searches the given directory for all the files that end with
	the given extender. Each file found is stuffed into fp. assumes fp is 
	allocated and initialized. returns true if successful.
*/

void muKillFileList( FileListPtr fp );
/* this routine releases all memory associated with a file list ptr. */

FileListPtr muSearchForModules( char *partialPath, char *ext );
/* this routine goes through each of the default module directories and
	searches for the files with the given ext. partialPath refers to an 
	optional partial path that starts from the module directories (ie 
	.../<ModuleDirectory>/<partialPath>/, where <partialPath>/ is the partial 
	path). This routine excludes any duplicate files. returns FileListPtr if 
	successful, else NULL.
*/

/* directory path creation utility routines */
void muGetAppModulePath( char *partialPath, char *fullPath );
/* this routine returns the application .app path for the searched modules.
	concats the passed partial path onto the app directory path.
*/

void muGetHomeModulePath( char *partialPath, char *fullPath );
/* this routine returns the home path for the searched modules. concats the
	passed partial path onto the home directory path.
*/

void muGetPublicModulePath( char *partialPath, char *fullPath );
/* this routine returns the public path for the searched modules. concats the
	passed partial path onto the public directory path.
*/

