/*
	nlClassEyes.h

	This class is the controller class of the Eyes NiteLite module.  Use this code as an 
	example of how to create a NiteLite module.

	You may freely copy, distribute, and reuse the code in this example.
	Metrosoft disclaims any warranty of any kind, expressed or  implied, as to its
	fitness for any particular use.
*/

#import <appkit/appkit.h>
#import <MetroTools/NiteLite/nlClassModule.h>

#define NEYES		6			/* # of animated eye frames */
#define NSOUNDS		2			/* # of nighttime sounds */
#define MAXEYES		15			/* max # of eyes on screen at one time */

/* # of blinks per set of eyes */
#define MINBLINKS	2
#define MAXBLINKS	5

/* timing values */
#define EYEANIMTIME		50			/* 50ms per frame */
#define EYEDELAY		1000		/* 1 sec */
#define SOUNDDELAY		2000		/* 2 sec */

/* display control record */
typedef struct
	{
		int			eyeIndex;		/* eye type */
		unsigned	lastBlink;		/* time of last animation */
		int			blinkCount;		/* # of blinks already done */
		int			blinkIndex;		/* where are we in the animated blink sequence? */
		BOOL		inBlink;		/* are we currently in a blink? */
		NXPoint		eyePos;			/* position of eyes */
	} EyeRec;
	
typedef struct
	{
		int			curNEyes;		/* # of eyes currentlu displayed */
		unsigned	lastSoundTime;	/* time of last nighttime sound */
		unsigned	lastEyeTime;	/* time of last eye creation */
		EyeRec		eyes[MAXEYES];	/* eye list */
	} EyesDispRec;
	
@interface nlClassEyes:nlClassModule
{
	/* interface items */
    id	countSlider;
    id	soundsCheck;
	
	/* display parameters */
	int			nEyes;
	BOOL		doSounds;
	
	/* display control vars */
	EyesDispRec	*dispData;
	id			soundList[NSOUNDS];
	id			eyes_a[NEYES];
	id			eyes_b[NEYES];
	id			eyes_c[NEYES];
}

/* interface methods */
- setEyesCount:sender;
- setSounds:sender;

/* support methods */
- loadSound:(int)index;
- loadEye:(int)index;

@end
