/*
	mtClassMetroSuction.h

	This class is the controller class of the MetroSuction tool.  Use this code as an 
	example of how to create a MetroTools tool.

	You may freely copy, distribute, and reuse the code in this example.
	Metrosoft disclaims any warranty of any kind, expressed or implied, as to its
	fitness for any particular use.
*/

#import <appkit/appkit.h>
#import <MetroTools/MTModule.h>

// columns of file list table
#define colFilename		0
#define	colType			1
#define colArchs		2

#define MAX_ARCHS		4		// number of CPU types we support (includes subtypes)
#define MAX_STRIP_ARCHS	3		// number of CPU types we strip (no subtypes)
#define ARCHNAMELEN		10
#define ARCHDESCLEN		50

#define	TEMPFILENAME	"/tmp/MetroSuction.tmp"

// app list
typedef struct
{
	char	fileName[MAXPATHLEN+1];		// name of app
	char	filePath[MAXPATHLEN+1];		// full path of app 
	BOOL	isFat;						// fat group?
	int		nArch;						// number of architectures
	int		cpuIndex[MAX_ARCHS];		// index into our main cpu list
	int		archIndex[MAX_ARCHS];		// index into our architecture list
} AppRec, *AppListPtr;

@interface mtClassMetroSuction:MTModule
{
	id		fileDrop;
	id		tableView;
	id		logView;
	id		logBox;
	id		splitView;
	id		archChecks;
	id		thinButton;
	id		clearButton;
	id		removeButton;
	id		stringTable;
	id		formatter;

	int		nApps,hostcpuIndex;
	AppListPtr theApps;
}

// IB methods
- doThin:sender;
- doClear:sender;
- doRemove:sender;
- selectArch:sender;

//	DBTableView data source (psuedo-delegate) methods
- (unsigned int)rowCount;
- getValueFor:identifier at:(unsigned int)aPosition into:aValue;

//	DBTableView delegate methods
- tableViewDidChangeSelection:aTableView;

// NXSplitView Delegate methods
- splitView:sender getMinY:(NXCoord *)minY maxY:(NXCoord *)maxY ofSubviewAt:(int)offset;
 
// File list management methods
- addFile:(char *)path;
- removeEntry:(int)index;
- clearList;

// Support methods
- (BOOL) examineExecutable:(char *)path:(AppRec *)appInfo;
- (int)findcpuIndex:(int) cpu_type;
- (int) findArchIndex:(int) cpu_type:(int) cpu_subtype;
- addToLog:(char *)str;
- addFileToLog:(char *)fileName;
- clearLog;
- validateControls;
- validateThinButton;

@end
