/*
	mtClassMetroSuction.m

	This class is the controller class of the MetroSuction tool.  Use this code as an 
	example of how to create a MetroTools tool.

	You may freely copy, distribute, and reuse the code in this example.
	Metrosoft disclaims any warranty of any kind, expressed or implied, as to its
	fitness for any particular use.
*/

#import "mtClassMetroSuction.h"

#include <sys/file.h>

#import <mach-o/arch.h>
#import <mach-o/loader.h>
#import <mach-o/fat.h>

#import <dbkit/DBTableView.h>
#import <dbkit/DBTableVector.h>
#import <dbkit/DBTextFormatter.h>
#import <dbkit/tableProtocols.h>

#import <MetroTools/MetroTools.h>
#import <MetroUtils/muFileDrop.h>

/*
	 IMPORTANT NOTE: Add CPU variations to the END of list!!! Add new major CPU types 
					(or architectures you want to split seperately)
					to the beginning of list and increment MAX_STRIP_ARCHS
*/

static int cpu_types[MAX_ARCHS] = { CPU_TYPE_MC680x0,
									CPU_TYPE_I386,
									CPU_TYPE_HPPA,
									CPU_TYPE_I386 };

static int cpu_subtypes[MAX_ARCHS] = { CPU_SUBTYPE_MC680x0_ALL,
									CPU_SUBTYPE_I386_ALL,
									CPU_SUBTYPE_HPPA_ALL,
									CPU_SUBTYPE_486 };

static char arch_names[MAX_ARCHS][ARCHNAMELEN+1] = { "m68k","i386","hppa","i486" };

static char arch_desc[MAX_ARCHS][ARCHDESCLEN+1] = { "Motorola 68k",
													"Intel 80x86",
													"HP-PA",
													"Intel 486" };

static BOOL strip_cpu[MAX_STRIP_ARCHS] = { YES, YES, YES };


@implementation mtClassMetroSuction

// MetroTools methods

- InitSelf
{
	const NXArchInfo	*archInfo;
	int i;

	nApps = 0;

	// get host's cpu type
	archInfo = NXGetLocalArchInfo();
	hostcpuIndex = [self findcpuIndex:archInfo->cputype];
	strip_cpu[hostcpuIndex] = NO;

	// create empty App list
	theApps = (AppListPtr)malloc(0);

	// setup split view
    [splitView addSubview:[tableView removeFromSuperview]];
    [splitView addSubview:[logBox removeFromSuperview]];
    [splitView display];

	// setup table view
    [[tableView setDataSource:self] setDelegate:self];
	formatter = [[DBTextFormatter alloc] init];
	for (i = colFilename; i <= colArchs; i++)
	{
	    [[[tableView columnAt:i] setFormatter:formatter] setIdentifier:(id)i];
		
	}

	// setup controls
	[self validateControls];

	// setup file drop
	[fileDrop setEnterFolders:YES];

	return	self;
}

- doActivate
{
	// enable drag-and-drop 
	[fileDrop setEnabled:YES];
	
	return	self;
}

- doDeactivate
{
	// disable drag-and-drop 
	[fileDrop setEnabled:NO];

	return	self;
}

- unload
{
	free(theApps);

	return	self;
}


// muFileDrop delegate methods

- (BOOL)muFDiconEntered: sender
{
	return YES;
}

- (BOOL)muFDprocessFile:(char *)name
{
	[self addFile:name];

	return YES;		
}

- (void)muFDfinished:sender;
{
	// we want window changes to appear all at once
	[[logView window] disableFlushWindow];
    
	// update the table
    [tableView reloadData:self];

	// update controls
	[self validateControls];

	// flush changes to screen
	[[[logView window] reenableFlushWindow] flushWindow];
}

// IB methods

- doThin:sender
{
	BOOL thinAll=NO,thinSelected=NO;
	int i,j,nToStrip,result;
	char cmdBuffer[MAXPATHLEN+128];

	[self clearLog];

	if (([tableView selectedRowCount] > 0) && ([tableView selectedRowCount] != nApps))
	{
		// see if user meant to do all the apps in the list or just the selected ones
		result = NXRunAlertPanel([stringTable valueForStringKey:"confirm"],
										[stringTable valueForStringKey:"thin selection warning"],
										[stringTable valueForStringKey:"thin all"], 
										[stringTable valueForStringKey:"thin selected"], 
										[stringTable valueForStringKey:"Cancel"]);
		if (result == NX_ALERTDEFAULT) thinAll = YES;
		else if (result == NX_ALERTALTERNATE) thinSelected = YES;
	}
	else
	{
		// verify that the user wants to proceed
		result = NXRunAlertPanel([stringTable valueForStringKey:"confirm"],
										[stringTable valueForStringKey:"thin warning"],
										[stringTable valueForStringKey:"OK"], 
										[stringTable valueForStringKey:"Cancel"], 0);
		if (result == NX_ALERTDEFAULT) thinAll = YES;
	}

	if (thinAll || thinSelected)
	{
		if (strip_cpu[hostcpuIndex])
		{
			// ensure user intends to remove support for native architecture
			result = NXRunAlertPanel([stringTable valueForStringKey:"warning"],
										[stringTable valueForStringKey:"thin host architecture"],
										[stringTable valueForStringKey:"OK"], 
										[stringTable valueForStringKey:"Cancel"], 0,
										arch_desc[hostcpuIndex]);
			if (result != NX_ALERTDEFAULT) return self;
		}

		//  tell user we're starting
		[self addToLog:(char *)
			[stringTable valueForStringKey:(thinAll ? "thinning" : "thinning selected")]];
		NXPing();

		// loop through list
		for (i=0; i<nApps; i++)
		{
			if (thinAll || ([tableView isRowSelected:i]))
			{
				[self addToLog:theApps[i].fileName];
				[self addToLog:"..."];
				if (!theApps[i].isFat)
					[self addToLog:(char *) [stringTable valueForStringKey:"already thin"]];
				else
				{
					// check to see that binary is writable
					if (access(theApps[i].filePath, W_OK))
					{
						// verify that the user wants to proceed
						result = NXRunAlertPanel([stringTable valueForStringKey:"error"],
										[stringTable valueForStringKey:"write permission error"],
										[stringTable valueForStringKey:"skip file"], 
										[stringTable valueForStringKey:"stop everything"], 0,
										theApps[i].fileName);

						if (result == NX_ALERTDEFAULT)
						{
							[self addToLog:(char *)[stringTable valueForStringKey:"skipped"]];
							continue;
						}
						else break;
					}

					// build lipo command
					sprintf(cmdBuffer, "lipo %s -output %s",
										theApps[i].filePath,theApps[i].filePath);
					nToStrip = 0;
					for (j=0; j<theApps[i].nArch; j++)
					{
						if (strip_cpu[theApps[i].cpuIndex[j]])
						{
							strcat(cmdBuffer," -remove ");
							strcat(cmdBuffer,arch_names[theApps[i].archIndex[j]]);
							nToStrip++;
						}
					}
					
					if (!nToStrip)
					{
						// nothing to do for this one
						[self addToLog:(char *)[stringTable valueForStringKey:"nothing"]];
						continue;
					}

					// check to see if binary would be empty
					if (nToStrip == theApps[i].nArch)
					{
						// verify that the user wants to proceed
						result = NXRunAlertPanel([stringTable valueForStringKey:"warning"],
										[stringTable valueForStringKey:"empty file"],
										[stringTable valueForStringKey:"skip file"], 
										[stringTable valueForStringKey:"stop everything"], 0,
										theApps[i].fileName);

						if (result == NX_ALERTDEFAULT)
						{
							[self addToLog:(char *)[stringTable valueForStringKey:"skipped"]];
							continue;
						}
						else break;
					}

					// do the actual liposuction
					strcat(cmdBuffer," >");
					strcat(cmdBuffer,TEMPFILENAME);
					strcat(cmdBuffer," 2>&1 \n");
					if(system(cmdBuffer))
					{
						// non-zero exit status == must report the errors
						[self addToLog:(char *)[stringTable valueForStringKey:"failed"]];
						[self addFileToLog:TEMPFILENAME];
						[self addToLog:"\n"];
						unlink(TEMPFILENAME);

						// verify that the user wants to proceed
						result = NXRunAlertPanel([stringTable valueForStringKey:"error"],
										[stringTable valueForStringKey:"lipo error"],
										[stringTable valueForStringKey:"skip file"], 
										[stringTable valueForStringKey:"stop everything"], 0,
										theApps[i].fileName);

						if (result == NX_ALERTDEFAULT) continue;
						else break;
					}
					unlink(TEMPFILENAME);
					[self addToLog:(char *)[stringTable valueForStringKey:"file done"]];

					// re-examine contents of file
					[self examineExecutable:theApps[i].filePath:&theApps[i]];
				}
			}
		}

		//  tell user we're done if we didn't abort
		if (i==nApps)
			[self addToLog:(char *)[stringTable valueForStringKey:"Done"]];
		else
			[self addToLog:(char *)[stringTable valueForStringKey:"stopped"]];
			NXPing();

		// update the table
    	[tableView reloadData:self];

		// and re-validate controls
		[self validateControls];
	}

	return self;
}

- doClear:sender
{
	// clear the data list structure
	[self clearList];

	// update the table
    [tableView reloadData:self];

	// update controls
	[self validateControls];

	return self;
}

- doRemove:sender
{
	int i;

	// remove selected items
	for (i=nApps-1; i>=0; i--)
		if ([tableView isRowSelected:i])
			[self removeEntry:i];

	// update the table
    [tableView reloadData:self];

	// update controls
	[self validateControls];

	return self;
}

- selectArch:sender
{
	id	cell;
	
	cell = [sender selectedCell];
	strip_cpu[[cell tag]] = [cell state];

	// enable or disable thin button
	[self validateThinButton];

	return self;
}

//	DBTableView data source (psuedo-delegate) methods

- (unsigned int)rowCount
{
    return nApps;
}

// DBTV sends this message when it needs to know what to display
- getValueFor:identifier at:(unsigned int)aPosition into:aValue
{
	char str[80];
	int n,i;

	switch ((int)identifier)
	{
		case colFilename:
			[aValue setStringValue:theApps[aPosition].fileName];
			break;
		case colType:
			[aValue setStringValue:
				[stringTable valueForStringKey:theApps[aPosition].isFat ? "fat" : "thin"]];
			break;
		case colArchs:
			if (n = theApps[aPosition].nArch)
			{
				if (theApps[aPosition].archIndex[0] != -1)
					strcpy(str, arch_desc[theApps[aPosition].archIndex[0]]);
				else
					strcpy(str, [stringTable valueForStringKey:"unknown"]);
				for (i=1; i<n; i++)
				{
					strcat(str, ", ");
					if (theApps[aPosition].archIndex[i] != -1)
						strcat(str, arch_desc[theApps[aPosition].archIndex[i]]);
					else
						strcat(str, [stringTable valueForStringKey:"unknown"]);
				}
				[aValue setStringValue:str];
			}
			else
				[aValue setStringValue:""];
			break;
	}
    return nil;
}

//	DBTableView delegate methods

- tableViewDidChangeSelection:aTableView
{
	unsigned int col;

	if ((col = [tableView selectedColumn]) != -1)
		[tableView deselectColumn:col];

	// update controls
	[self validateControls];

	return self;
}

// NXSplitView Delegate methods

// Constrain the divider
- splitView:sender 
    getMinY:(NXCoord *)minY 	// in the split view's flipped coordinates
    maxY:(NXCoord *)maxY
    ofSubviewAt:(int)offset     // offset = 0 is the first divider
{
    *minY = 42.0;
	*maxY = 154.0;

    return self;
}

// App list management methods

- addFile:(char *)path
{
	AppRec		appInfo;
	size_t		nBytes;
 	char 		*p;
	int			i;

	if (![self examineExecutable:path:&appInfo])
		return self;

	// check to see if it's already in the list
	for (i=0; i<nApps; i++)
	{
		if (!strcmp(path,theApps[i].filePath))
		{
			return self;
		}
	}

	// strip down to name only
	for (p=&path[0]+strlen(path); p>=&path[0]; p--)
		if (*p=='/') break;
	p++;
	
	nBytes = sizeof(AppRec) * (nApps+1);
	theApps = (AppListPtr)realloc( theApps, nBytes );
	if( theApps )
	{
		strcpy(theApps[nApps].filePath,path);
		strcpy(theApps[nApps].fileName,p);
		theApps[nApps].isFat = appInfo.isFat;
		theApps[nApps].nArch = appInfo.nArch;
		for (i=0; i<appInfo.nArch; i++)
		{
			theApps[nApps].cpuIndex[i] = appInfo.cpuIndex[i];
			theApps[nApps].archIndex[i] = appInfo.archIndex[i];
		}
		// indicate one more App
    	nApps++;
	}
	else
	{
		theApps = (AppListPtr)malloc(0);
		nApps = 0;
	}
	
	return self;
}

- removeEntry:(int)index
{
	int			length;
	size_t		nBytes;
	
	if( nApps < 1 || index >= nApps )
		return self;
		
	nApps--;
	if( index < nApps )
	{
		length = (nApps - index) * sizeof(AppRec);
		bcopy( &(theApps[index+1]), &(theApps[index]), length );
	}
	
	nBytes = sizeof(AppRec) * nApps;
	theApps = (AppListPtr)realloc( theApps, nBytes );
	if( !theApps )
	{
		theApps = (AppListPtr)malloc(0);
		nApps = 0;
	}
	return self;
}

- clearList
{
	theApps = (AppListPtr)realloc( theApps, 0 );
	nApps = 0;

	return self;
}

// Support methods

- (BOOL) examineExecutable:(char *)path:(AppRec *)appInfo
{
	NXStream	*s;
	int 		data,cpu_type,cpu_subtype,i;
	long		l_data;
	BOOL		isExecutable=NO,swapped=NO;

	// if executable bit isn't set or we can't read the file, don't bother examining file header
	if (access(path, X_OK) || access(path, R_OK))
		return NO;

	// open the file and examine mach-o header
	s = NXMapFile( path, NX_READONLY );
	if (s)
	{
		// get magic number
		if (NXRead(s,&data,sizeof(int)) == sizeof(int))
		{
			if (data==MH_MAGIC || data==FAT_MAGIC || data==MH_CIGAM || data==FAT_CIGAM)
			{
				appInfo->nArch = 0;
				if (data==MH_CIGAM || data==FAT_CIGAM) swapped = YES;

				if (data==FAT_MAGIC || data==FAT_CIGAM)
				{
					appInfo->isFat = YES;

					// get number of architectures
					if (NXRead(s,&data,sizeof(int)) == sizeof(int))
					{
						appInfo->nArch = MIN((swapped ? NXSwapLong(data) : data), MAX_ARCHS);
						for (i=0; i<appInfo->nArch; i++)
						{
							// get cpu type
							if ((NXRead(s,&cpu_type,sizeof(int)) == sizeof(int))
								&& (NXRead(s,&cpu_subtype,sizeof(int)) == sizeof(int)))
							{
								appInfo->archIndex[i] =
									[self findArchIndex
										: (swapped ? NXSwapLong(cpu_type) : cpu_type)
										: (swapped ? NXSwapLong(cpu_subtype) : cpu_subtype)];

								appInfo->cpuIndex[i] =
									[self findcpuIndex
										: (swapped ? NXSwapLong(cpu_type) : cpu_type)];

								// skip rest of record
								NXRead(s,&l_data,sizeof(long));
								NXRead(s,&l_data,sizeof(long));
								NXRead(s,&l_data,sizeof(long));
							}
							else
							{
								appInfo->nArch = i;
								break;
							}
						}
					}
				}
				else
				{
					appInfo->isFat = NO;

					// get cpu type
					if ((NXRead(s,&cpu_type,sizeof(int)) == sizeof(int))
						&& (NXRead(s,&cpu_subtype,sizeof(int)) == sizeof(int)))
						{
							appInfo->archIndex[0] =
								[self findArchIndex
									: (swapped ? NXSwapLong(cpu_type) : cpu_type)
									: (swapped ? NXSwapLong(cpu_subtype) : cpu_subtype)];

								appInfo->cpuIndex[0] =
									[self findcpuIndex
										: (swapped ? NXSwapLong(cpu_type) : cpu_type)];
								
								appInfo->nArch = 1;
						}
				}
				isExecutable = YES;
			}
		}
		NXCloseMemory( s, NX_FREEBUFFER );
	}

	return isExecutable;
}

- (int)findcpuIndex:(int) cpu_type
{
	int	cpuIndex=-1, i;

	for (i=0; i<MAX_STRIP_ARCHS; i++)			// look it up in our table
		if (cpu_type == cpu_types[i])
		{
			cpuIndex = i;
			break;
		}

	return cpuIndex;
}

- (int)findArchIndex:(int) cpu_type:(int) cpu_subtype
{
	int	archIndex=-1, i;
	const NXArchInfo	*archInfo;

	for (i=0; i<MAX_ARCHS; i++)			// look it up in our table
		if ((cpu_type == cpu_types[i]) && (cpu_subtype == cpu_subtypes[i]))
		{
			archIndex = i;
			break;
		}
	if (i == MAX_ARCHS)
	{
			archInfo = NXGetArchInfoFromCpuType(cpu_type, MAX_STRIP_ARCHS);
			NXRunAlertPanel([stringTable valueForStringKey:"warning"],
								[stringTable valueForStringKey:"unknown cpu"],
													0, 0, 0, archInfo->description);
	}

	return archIndex;
}

- addToLog:(char *)str
{
	int anchor;
		
	[[logView window] disableFlushWindow];

	anchor = [logView textLength];
	if (anchor <= 0)
		[logView setText:str];
	else
    	[[logView setSel:anchor:anchor] replaceSel:str];
    [[logView scrollSelToVisible] hideCaret];

	[[[logView window] reenableFlushWindow] flushWindow];

	return self;
}

- addFileToLog:(char *)fileName
{
	char *str ;
	struct stat statBuf ;
	int fd ;

	if (!(fd = open(fileName,O_RDONLY)))
    	return self ;

	fstat(fd,&statBuf) ;
	str = malloc(statBuf.st_size + 1) ;
	read(fd,str,statBuf.st_size) ;
	str[statBuf.st_size] = '\0' ;
	[self addToLog:str];
	close(fd) ;
	free(str) ;

	return self;
}

- clearLog
{
	[[logView window] disableFlushWindow];

    [logView setSel:0:[logView textLength]];
    [logView replaceSel:""];
    [logView scrollSelToVisible];
    [logView hideCaret];

	[[[logView window] reenableFlushWindow] flushWindow];

	return self;
}

- validateControls
{
	int i,j;
	BOOL hasFat = NO, hasArch[MAX_STRIP_ARCHS] = { NO, NO, NO };

	// we want window changes to appear all at once
	[[logView window] disableFlushWindow];

	// find out which architectures are represented in list
	if (nApps)
	{
		for (i=0; i<nApps; i++)
		{
			if (theApps[i].isFat)
			{
				hasFat = YES;
				for (j=0; j<theApps[i].nArch; j++)
					hasArch[theApps[i].cpuIndex[j]] = YES;
			}
		}
	}

	// setup architecture buttons
	for (i=0;i<MAX_STRIP_ARCHS;i++)
	{
		if (i != hostcpuIndex)
			strip_cpu[i] = hasArch[i];
		else if (!hasArch[i])
			strip_cpu[i] = NO;

		[[archChecks cellAt:i:0] setState:strip_cpu[i]];
		[[archChecks cellAt:i:0] setEnabled:(hasArch[i] ? YES : NO)];
	}

	// enable and disable other controls
	[self validateThinButton];
	[clearButton setEnabled:((nApps==0) ? NO : YES)];
	[removeButton setEnabled:(([tableView selectedRowCount] > 0) ? YES : NO)];
   
	// flush changes to screen
	[[[logView window] reenableFlushWindow] flushWindow];

	return self;
}

- validateThinButton
{
	int i;
	BOOL somethingToThin = FALSE;

	for (i=0; i<MAX_STRIP_ARCHS; i++)
	{
		if (strip_cpu[i])
		{
			somethingToThin = YES;
			break;
		}
	}
	[thinButton setEnabled:((somethingToThin) ? YES : NO)];

	return self;
}

@end
